@@ -1,5 +1,5 @@
 # Repository: https://gitlab.com/quantify-os/quantify-core
-# Licensed according to the LICENCE file on the master branch
+# Licensed according to the LICENCE file on the main branch
 """
 Module containing an education example of an analysis subclass.
 
@@ -38,7 +38,7 @@ class CosineAnalysis(ba.BaseAnalysis):
         """
         # create a fitting model based on a cosine function
         model = CosineModel()
-        guess = model.guess(self.dataset.y0.values)
+        guess = model.guess(self.dataset.y0.values, x=self.dataset.x0.values)
         result = model.fit(
             self.dataset.y0.values, x=self.dataset.x0.values, params=guess
         )
